#ifndef __TW_IOCTL_H__
#define __TW_IOCTL_H__

/***************************************************************
Copyright 2002-2004 3M. All rights reserved.

Program Name:		MtsTch.sys
Module Name:		tw_ioctl.h

Summary:		Interface to Microtouch touch screens drivers (IOCTLs)

Description:
	This Linux touchscreen driver module contains definitions of
   IOCTL commands and Structures to pass applicable parameters to driver.

Notes:

***************************************************************/


/*USE #DEFINES INSTEAD OF ENUMS   */
#define LAST_BYTE_IN_BUFFER 255

typedef enum
{
   TW_DRIVER_CMD = 0,
   TW_TS_CMD = 1,
   TW_TS_CMD_SERIAL_ONLY = 2,
   TW_TS_CMD_USB_ONLY = 3,
   TW_TS_READ = 4,
   TW_TS_WRITE = 5,
   TW_TS_SERIAL_GENERIC = 6,     /* app builds a command string to send to the controllers  */
   TW_TS_USB_GENERIC = 7,        /* app builds a USB struct to send to the controllers  */
   /* add new ones here  */
   TW_TS_DUMMY

} TW_IOCTL_CMDS;


/* List of Commands used for driver functionality */
typedef enum
{
   TW_DRIVER_CMD_NOP = 0,

   TW_DRIVER_CMD_WHO_ARE_YOU = 1,
   TW_DRIVER_CMD_GET_PARAM = 2,
   TW_DRIVER_CMD_SET_PARAM = 3,
   TW_DRIVER_CMD_GET_ALL_PARAMS = 4,
   TW_DRIVER_CMD_SET_ALL_PARAMS = 5,
   TW_DRIVER_CMD_CANCEL_IOCTL = 6,
   TW_DRIVER_CMD_IS_DRIVER_INIT_DONE = 7,
   TW_DRIVER_CMD_GET_TOUCHSCREEN_COUNT = 8,
   TW_DRIVER_CMD_SET_SURROUND_BUTTON = 9,
   TW_DRIVER_CMD_GET_LAST_TOUCH_DOWN = 10,
   TW_DRIVER_CMD_GET_LAST_LIFT_OFF = 11,
   TW_DRIVER_CMD_GET_LAST_DOUBLE_TOUCH = 12,
   TW_DRIVER_CMD_RESET = 13,
   TW_DRIVER_CMD_GET_CONFIGURATION = 14,
   /* add new ones here */

   TW_DRIVER_CMD_DUMMY

} TW_DRIVER_CMDS;


/* list of commands to the TouchScreen controller (no reply expected)  */
typedef enum
{
   TW_TS_CMD_NOP = 0,

   TW_TS_CMD_RESET,
   TW_TS_CMD_RESTORE_DEFAULTS,
   TW_TS_CMD_SET_PEN_FINGER,
   TW_TS_CMD_SET_WAKE_ON_TOUCH,
   TW_TS_CMD_START_CALIBRATE,
   TW_TS_CMD_START_RAW_DATA,
   TW_TS_CMD_FLUSH_RAW_DATA,
   TW_TS_CMD_REBOOT_RAM,
   TW_TS_CMD_REBOOT_ROM,
   TW_TS_CMD_REBOOT_EEPROM,
   TW_TS_CMD_DOWNLOAD_EEPROM,
   TW_TS_CMD_DOWNLOAD_RAM,
   TW_TS_CMD_DOWNLOAD_BOTH,
   TW_TS_CMD_DOWNLOAD_END,
   TW_TS_CMD_FORMAT_MERIT,
   /* add new ones here */

   TW_TS_CMD_DUMMY

} TW_TS_CMDS;

/* list of read commands to the TouchScreen controller */
typedef enum
{
   TW_TS_READ_NOP = 0,

   TW_TS_READ_STATUS,
   TW_TS_READ_VERSION,
   TW_TS_READ_RAW_DATA,
   TW_TS_READ_CABLE_NOVRAM,
   TW_TS_READ_CONTROLLER_NOVRAM,
   TW_TS_READ_EEPROM,
   TW_TS_READ_EEPROM_STATUS,
   TW_TS_READ_CONTROLLER_ID,
   TW_TS_READ_IS_EXII,
   TW_TS_READ_IS_FIXED_CODE,
	TW_TS_READ_PING,
   /* add new ones here */

   TW_TS_READ_DUMMY,


} TW_TS_READS;


/* list of write commands to the TouchScreen controller  */
typedef enum
{
   TW_TS_WRITE_NOP = 0,

   TW_TS_WRITE_DOWNLOAD_DATA,
   TW_TS_WRITE_CABLE_NOVRAM,
   TW_TS_WRITE_CONTROLLER_NOVRAM,
   TW_TS_WRITE_EEPROM,
   /* add new ones here */

   TW_TS_WRITE_DUMMY

} TW_TS_WRITES;

#define FORMAT_TABLET      0
#define FORMAT_RAW         1
#define FORMAT_HEX         2
#define FORMAT_DECIMAL     3
#define FORMAT_MULTIPLAYER 4
/* list of commands specific to serial controllers */
typedef enum
{
   TW_TS_CMD_SERIAL_ONLY_NOP = 0,

	TW_TS_CMD_SERIAL_ONLY_AUTO_BAUD,          /* (enable, disable) */
	TW_TS_CMD_SERIAL_ONLY_COM_SETTINGS,       /* (Parity, Data & Stop Bits, Baud) */
	TW_TS_CMD_SERIAL_ONLY_COM_SETTINGS_LOCK,  /* locks params in TW Serial Controller */
	TW_TS_CMD_SERIAL_ONLY_DATA_FORMAT,        /* (Tablet, Raw, Hex, Decimal, Multiplayer) */
	TW_TS_CMD_SERIAL_ONLY_DATA_MODE,          /* (Stream, Status, Polled, Point, Inactive) */
   /* add new ones here */

   TW_TS_CMD_SERIAL_ONLY_DUMMY

} TW_TS_CMD_SERIAL_ONLYS;



typedef struct
{
      unsigned char     m_ucId[4];      /* identifies particular touchscreen */
      unsigned long     m_ulSubCommand; /* "minor command */
      unsigned long     m_ulData;       /* example: enable/disable, true/false, modea mode b */
      unsigned long     m_ulDataSize;   /* sizeof appropriate structure for particular ioctl command */
      unsigned char     m_pBuffer[LAST_BYTE_IN_BUFFER+1]; /* to be casted to appropriate structure for particular ioctl command */
}  TW_IOCTL_STRUCT;

typedef struct
{
   char m_cStatus;                  /* Status byte */
   char m_cXTouch [2];              /* X coordinate of touch (raw) */
   char m_cYTouch [2];              /* Y coordinate of touch (raw) */
   char m_cLevelPlayerA [2];        /* Touch level of player A */
   char m_cLevelPlayerB [2];        /* Touch level of player B */
   char m_cLevelTouch [2];          /* Level of touch */
} TW_IOCTL_DC_LAST_DOUBLE_TOUCH;

typedef struct
{
   char m_cStatus;                  /* Status byte */
   char m_cXTouch [2];              /* X coordinate of touch (raw) */
   char m_cYTouch [2];              /* Y coordinate of touch (raw) */
   char m_cLevelPlayerA [2];        /* Touch level of player A */
   char m_cLevelPlayerB [2];        /* Touch level of player B */
   char m_cLevelTouch [2];          /* Level of touch */
   unsigned short m_nX;             /* X coordinate of touch (conditioned) */
   unsigned short m_nY;             /* Y coordinate of touch (conditioned) */
} TW_IOCTL_DC_LAST_DOUBLE_TOUCH_V2;

#ifdef MS_WINDOWS

/* Pick a custom device type number: */
#define FILE_DEVICE_TOUCH 0x1A

#define IOCTL_TW_DRIVER_CMD         CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_DRIVER_CMD,         METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_CMD             CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_CMD,             METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_CMD_SERIAL_ONLY CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_CMD_SERIAL_ONLY, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_CMD_USB_ONLY    CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_CMD_USB_ONLY,    METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_READ            CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_READ,            METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_WRITE           CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_WRITE,           METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_SERIAL_GENERIC  CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_SERIAL_GENERIC,  METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_TW_TS_USB_GENERIC     CTL_CODE(FILE_DEVICE_TOUCH, 0x800+TW_TS_USB_GENERIC,     METHOD_BUFFERED, FILE_ANY_ACCESS)

#else

/* Pick a magic number:  */
#define FILE_DEVICE_TOUCH 'k'

#define IOCTL_TW_DRIVER_CMD         _IOWR(FILE_DEVICE_TOUCH, TW_DRIVER_CMD,         TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_CMD             _IOWR(FILE_DEVICE_TOUCH, TW_TS_CMD,             TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_CMD_SERIAL_ONLY _IOWR(FILE_DEVICE_TOUCH, TW_TS_CMD_SERIAL_ONLY, TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_CMD_USB_ONLY    _IOWR(FILE_DEVICE_TOUCH, TW_TS_CMD_USB_ONLY,    TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_READ            _IOWR(FILE_DEVICE_TOUCH, TW_TS_READ,            TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_WRITE           _IOWR(FILE_DEVICE_TOUCH, TW_TS_WRITE,           TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_SERIAL_GENERIC  _IOWR(FILE_DEVICE_TOUCH, TW_TS_SERIAL_GENERIC,  TW_IOCTL_STRUCT)
#define IOCTL_TW_TS_USB_GENERIC     _IOWR(FILE_DEVICE_TOUCH, TW_TS_USB_GENERIC,     TW_IOCTL_STRUCT)

#define TW_IOC_MAXNR TW_TS_DUMMY

#endif /* ifdef MS_WINDOWS */
#endif  /* __TW_IOCTL_H__ */
